@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -17,12 +17,14 @@
 #ifndef BZF_ERROR_HANDLER_H
 #define	BZF_ERROR_HANDLER_H
 
-#ifdef _MSC_VER
-#pragma warning( 4: 4786 )
-#endif
+// common header first
+#include "common.h"
 
+// system headers
 #include <vector>
 #include <string>
+
+// local implementation headers
 #include "common.h"
 #include "Bundle.h"
 
