@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -18,25 +18,26 @@
 #define	BZF_COMMON_H
 
 // this should always be the very FIRST header
-#include "config.h"
+#ifdef _DEVCPP //the Dev-C++ build is acting very stubborn; this is (hopefully) -temporary-
+# include_next "config.h"
+#else
+# include "config.h"
+#endif
 
-#if (_MSC_VER)
-// turn off bogus `this used in base member initialization list'
-#  pragma warning(disable: 4786)
-#  pragma warning(disable: 4503)
-#  pragma warning(disable: 4355)
-#endif // _MSC_VER
+#ifdef _WIN32
+#include "win32.h"
+#define _FD_SET(fd, set) FD_SET((unsigned int)fd, set)
+#else
+#define _FD_SET(fd, set) FD_SET(fd, set)
+#endif
 
-#include <string>
-#include <sstream>
-#include <vector>
-#include <stdarg.h>
 #include <stdio.h>
-
-#include "bzfio.h"
+#include <stdlib.h> //needed for bzfrand
+#include <math.h>
 
 extern int debugLevel;
-// Like verbose debug messages?
+// Like verbose debug messages? level 0 for development only
+#define DEBUG0 formatDebug
 #define DEBUG1 if (debugLevel >= 1) formatDebug
 #define DEBUG2 if (debugLevel >= 2) formatDebug
 #define DEBUG3 if (debugLevel >= 3) formatDebug
@@ -49,67 +50,98 @@ extern int debugLevel;
 
 // seven places of precision is pretty safe, so something less precise
 #if defined(FLT_EPSILON)
-#	define ZERO_TOLERANCE FLT_EPSILON
+#  define ZERO_TOLERANCE FLT_EPSILON
 #else
-#	define ZERO_TOLERANCE 1.0e-06f
+#  define ZERO_TOLERANCE 1.0e-06f
 #endif
 
 // Might we be BSDish? sys/param.h has BSD defined if so
-#if (defined(__unix__) || defined(unix) || defined(__APPLE__)) && !defined(USG)
-#include <sys/param.h>
+#ifdef HAVE_SYS_PARAM_H
+#  include <sys/param.h>
 #endif
 
-#if defined HAVE_STRICMP
-#define strcasecmp _stricmp
-#define strncasecmp _strnicmp
-#define vsnprintf _vsnprintf
+#ifdef HAVE__STRICMP
+#  define strcasecmp _stricmp
+#endif
+#ifdef HAVE__STRNICMP
+#  define strncasecmp _strnicmp
+#endif
+#if !defined(HAVE_VSNPRINTF)
+#  ifdef HAVE__VSNPRINTF
+#    define vsnprintf _vsnprintf
+#  else
+#    define vsnprintf(buf, size, fmt, list) vsprintf(buf, fmt, list)
+#  endif
 #endif
-
-#include <assert.h>
-#include <stddef.h>
 
 // some platforms don't have float versions of the math library
-#if defined(_old_linux_) || defined(__APPLE__) || defined(sun)
-#define	asinf		(float)asin
-#define	atanf		(float)atan
-#define	atan2f		(float)atan2
-#define	cosf		(float)cos
-#define	expf		(float)exp
-#if !defined(__APPLE__)
-#define	fabsf		(float)fabs
-#define	floorf		(float)floor
-#define	fmodf		(float)fmod
+#ifndef HAVE_ASINF
+#  define	asinf		(float)asin
 #endif
-#define	hypotf		(float)hypot
-#define	logf		(float)log
-#define	powf		(float)pow
-#define	sinf		(float)sin
-#define	sqrtf		(float)sqrt
-#define	tanf		(float)tan
+#ifndef HAVE_ATAN2F
+#  define	atan2f		(float)atan2
+#endif
+#ifndef HAVE_ATANF
+#  define	atanf		(float)atan
+#endif
+#ifndef HAVE_COSF
+#  define	cosf		(float)cos
+#endif
+#ifndef HAVE_EXPF
+#  define	expf		(float)exp
+#endif
+#ifndef HAVE_FABSF
+#  define	fabsf		(float)fabs
+#endif
+#ifndef HAVE_FLOORF
+#  define	floorf		(float)floor
+#endif
+#ifndef HAVE_FMODF
+#  define	fmodf		(float)fmod
+#endif
+#ifndef HAVE_HYPOTF
+#  define	hypotf		(float)hypot
+#endif
+#ifndef HAVE_LOGF
+#  define	logf		(float)log
+#endif
+#ifndef HAVE_POWF
+#  define	powf		(float)pow
+#endif
+#ifndef HAVE_SINF
+#  define	sinf		(float)sin
+#endif
+#ifndef HAVE_SQRTF
+#  define	sqrtf		(float)sqrt
+#endif
+#ifndef HAVE_TANF
+#  define	tanf		(float)tan
 #endif
 
 // random number stuff
-#include <stdlib.h>
 #define bzfrand()	((double)rand() / ((double)RAND_MAX + 1.0))
 #define bzfsrand(_s)	srand(_s)
 
+#if !defined(_WIN32)
 
-#if !defined(_WIN32) && !defined(__APPLE__)
-
-#ifndef BSD
-#  ifndef __BEOS__
+#ifndef __BEOS__
+#  ifdef HAVE_VALUES_H
 #    include <values.h>
-#  else
-#    include <limits.h>
+#  endif
+#else
+#  include <limits.h>
 /* BeOS: FIXME */
-#    define MAXSHORT SHORT_MAX
-#    define MAXINT INT_MAX
-#    define MAXLONG LONG_MAX
-#  endif /* __BEOS__ */
-#endif /* BSD */
+#  define MAXSHORT SHORT_MAX
+#  define MAXINT INT_MAX
+#  define MAXLONG LONG_MAX
+#endif /* __BEOS__ */
 
 #include <sys/types.h>
 
+#ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#endif
+
 #if defined(__linux) || (defined(__sgi) && !defined(__INTTYPES_MAJOR))
 typedef u_int16_t	uint16_t;
 typedef u_int32_t	uint32_t;
@@ -126,7 +158,7 @@ typedef uint_t		uint32_t;
 
 typedef unsigned char	uint8_t;
 
-#if defined( macintosh ) || defined( __BEOS__ )
+#if defined( __BEOS__ )
 
 // missing constants
 
@@ -145,11 +177,6 @@ typedef unsigned char	uint8_t;
 // need some integer types
 #  include <inttypes.h>
 
-// my own strcasecmp, missing in MSL
-#  ifdef __MWERKS__
-#    include "strcasecmp.h"
-#  endif
-
 #  ifndef setenv
 #    define setenv(a,b,c)
 #  endif
@@ -157,58 +184,7 @@ typedef unsigned char	uint8_t;
 #  ifndef putenv
 #    define putenv(a)
 #  endif
-#endif /* defined( macintosh ) || defined( __BEOS__ ) */
-
-/* definitions specific to Windows */
-#ifdef _WIN32
-
-// missing float math functions
-#  ifndef __MINGW32__
-#    define hypotf	(float)hypot
-#    define snprintf	_snprintf
-#  endif /* __MINGW32__ */
-
-// missing constants
-#  ifndef MAXFLOAT
-#    define	MAXFLOAT	3.402823466e+38f
-#  endif
-#  ifndef M_PI
-#    define	M_PI		3.14159265358979323846f
-#  endif
-#  ifndef M_SQRT1_2
-#    define	M_SQRT1_2	0.70710678118654752440f
-#  endif
-
-// missing types
-#  ifndef int16_t
-typedef signed short	int16_t;
-#  endif
-
-#  ifndef uint16_t
-typedef unsigned short	uint16_t;
-#  endif
-
-#  ifndef int32_t
-typedef signed int	int32_t;
-#  endif
-
-#  ifndef uint32_t
-typedef unsigned int	uint32_t;
-#  endif
-
-// there is no sigpipe in Windows
-#  ifndef SIGINT
-#    define SIGPIPE SIGINT
-#  endif
-
-/* VC includes this in config.h, MinGW can't
-   The right fix is a win32.h file, like 1.11 */
-#  ifdef __MINGW32__
-#    include <windows.h>
-#  endif
-
-#endif /* _WIN32 */
-
+#endif /* defined( __BEOS__ ) */
 
 #ifdef countof
 #  undef countof
@@ -226,4 +202,3 @@ typedef unsigned int	uint32_t;
 // indent-tabs-mode: t ***
 // End: ***
 // ex: shiftwidth=2 tabstop=8
-
