@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -13,21 +13,27 @@
 #ifndef __CUSTOMPYRAMID_H__
 #define __CUSTOMPYRAMID_H__
 
-// bzfs-specific headers
+/* interface header */
 #include "WorldFileObstacle.h"
+
+/* local interface header */
 #include "WorldInfo.h"
 
 
 class CustomPyramid : public WorldFileObstacle {
   public:
     CustomPyramid();
-    virtual void write(WorldInfo*) const;
+    virtual bool read(const char *cmd, std::istream& input);
+    virtual void writeToGroupDef(GroupDefinition*) const;
+
+  private:
+    bool flipZ;
 };
 
 #endif  /* __CUSTOMPYRAMID_H__ */
 
 // Local variables: ***
-// mode:C++ ***
+// mode: C++ ***
 // tab-width: 8 ***
 // c-basic-offset: 2 ***
 // indent-tabs-mode: t ***
