@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -14,27 +14,35 @@
 #pragma warning(4:4503)
 #endif
 
-
+// interface header
 #include "StateDatabase.h"
-#include "ErrorHandler.h"
+
+// system headers
 #include <assert.h>
 #include <ctype.h>
 #include <stack>
 #include <set>
 #include <iostream>
 #include <math.h>
+#include <string>
+
+// local implementation headers
+#include "ErrorHandler.h"
+#include "TextUtils.h"
 
 #if defined(DEBUG) || defined(_DEBUG)
-#include <TimeKeeper.h> // only for _debugLookups()
+// headers needed only for _debugLookups()
+#include <map>
+#include "TimeKeeper.h"
 
 void	_debugLookups(const std::string &name)
 {
   if (!BZDB.getDebug())
     return;
-  
+
   typedef std::map<std::string,int> EvalCntMap;
   static const float interval = 20.0f;
-  
+
   /* This bit of nastyness help debug BDZB->eval accesses sorted from worst to best*/
   static EvalCntMap cnts;
   static TimeKeeper last = TimeKeeper::getCurrent();
@@ -72,45 +80,74 @@ template <>
 StateDatabase* Singleton<StateDatabase>::_instance = (StateDatabase*)0;
 
 
+const std::string StateDatabase::BZDB_AGILITYADVEL	= std::string("_agilityAdVel");
+const std::string StateDatabase::BZDB_AGILITYTIMEWINDOW	= std::string("_agilityTimeWindow");
+const std::string StateDatabase::BZDB_AGILITYVELDELTA	= std::string("_agilityVelDelta");
 const std::string StateDatabase::BZDB_ANGLETOLERANCE    = std::string("_angleTolerance");
-const std::string StateDatabase::BZDB_ANGULARAD         = std::string("_angularAd");
-const std::string StateDatabase::BZDB_BOXHEIGHT         = std::string("_boxHeight");
+const std::string StateDatabase::BZDB_ANGULARAD		= std::string("_angularAd");
+const std::string StateDatabase::BZDB_AVENUESIZE	= std::string("_avenueSize");
+const std::string StateDatabase::BZDB_BASESIZE		= std::string("_baseSize");
+const std::string StateDatabase::BZDB_BOXBASE		= std::string("_boxBase");
+const std::string StateDatabase::BZDB_BOXHEIGHT		= std::string("_boxHeight");
 const std::string StateDatabase::BZDB_BURROWDEPTH       = std::string("_burrowDepth");
 const std::string StateDatabase::BZDB_BURROWSPEEDAD     = std::string("_burrowSpeedAd");
 const std::string StateDatabase::BZDB_BURROWANGULARAD   = std::string("_burrowAngularAd");
+const std::string StateDatabase::BZDB_COLDETDEPTH       = std::string("_coldetDepth");
+const std::string StateDatabase::BZDB_COLDETELEMENTS    = std::string("_coldetElements");
+const std::string StateDatabase::BZDB_CULLDEPTH		= std::string("_cullDepth");
+const std::string StateDatabase::BZDB_CULLELEMENTS      = std::string("_cullElements");
+const std::string StateDatabase::BZDB_CULLOCCLUDERS     = std::string("_cullOccluders");
+const std::string StateDatabase::BZDB_DISABLEBOTS       = std::string("_disableBots");
 const std::string StateDatabase::BZDB_EXPLODETIME       = std::string("_explodeTime");
 const std::string StateDatabase::BZDB_FLAGALTITUDE      = std::string("_flagAltitude");
-const std::string StateDatabase::BZDB_FLAGHEIGHT        = std::string("_flagHeight");
-const std::string StateDatabase::BZDB_FLAGPOLEWIDTH      = std::string("_flagPoleWidth");
+const std::string StateDatabase::BZDB_FLAGEFFECTTIME    = std::string("_flagEffectTime");
+const std::string StateDatabase::BZDB_FLAGHEIGHT	= std::string("_flagHeight");
+const std::string StateDatabase::BZDB_FLAGPOLEWIDTH     = std::string("_flagPoleWidth");
 const std::string StateDatabase::BZDB_FLAGPOLESIZE      = std::string("_flagPoleSize");
-const std::string StateDatabase::BZDB_FLAGRADIUS        = std::string("_flagRadius");
-const std::string StateDatabase::BZDB_GMISSILEANG	= std::string("_gMissileAng");
-const std::string StateDatabase::BZDB_GMISSILEADLIFE    = std::string("_gMissileAdLife");
+const std::string StateDatabase::BZDB_FLAGRADIUS	= std::string("_flagRadius");
+const std::string StateDatabase::BZDB_FRICTION		= std::string("_friction");
+const std::string StateDatabase::BZDB_GMACTIVATIONTIME	= std::string("_gmActivationTime");
+const std::string StateDatabase::BZDB_GMADLIFE		= std::string("_gmAdLife");
+const std::string StateDatabase::BZDB_GMTURNANGLE	= std::string("_gmTurnAngle");
 const std::string StateDatabase::BZDB_GRAVITY		= std::string("_gravity");
+const std::string StateDatabase::BZDB_HANDICAPSCOREDIFF = std::string("_handicapScoreDiff");
+const std::string StateDatabase::BZDB_HANDICAPVELAD	= std::string("_handicapVelAd");
+const std::string StateDatabase::BZDB_HANDICAPANGAD	= std::string("_handicapAngAd");
+const std::string StateDatabase::BZDB_HANDICAPSHOTAD	= std::string("_handicapShotAd");
 const std::string StateDatabase::BZDB_IDENTIFYRANGE	= std::string("_identifyRange");
 const std::string StateDatabase::BZDB_JUMPVELOCITY	= std::string("_jumpVelocity");
 const std::string StateDatabase::BZDB_LASERADVEL	= std::string("_laserAdVel");
-const std::string StateDatabase::BZDB_LASERADRATE 	= std::string("_laserAdRate");
-const std::string StateDatabase::BZDB_LASERADLIFE 	= std::string("_laserAdLife");
+const std::string StateDatabase::BZDB_LASERADRATE	= std::string("_laserAdRate");
+const std::string StateDatabase::BZDB_LASERADLIFE	= std::string("_laserAdLife");
+const std::string StateDatabase::BZDB_LATITUDE		= std::string("_latitude");
 const std::string StateDatabase::BZDB_LOCKONANGLE	= std::string("_lockOnAngle");
-const std::string StateDatabase::BZDB_LRADRATE   	= std::string("_lRAdRate");
+const std::string StateDatabase::BZDB_LONGITUDE		= std::string("_longitude");
+const std::string StateDatabase::BZDB_LRADRATE		= std::string("_lRAdRate");
+const std::string StateDatabase::BZDB_MAXBUMPHEIGHT     = std::string("_maxBumpHeight");
 const std::string StateDatabase::BZDB_MAXFLAGGRABS      = std::string("_maxFlagGrabs");
 const std::string StateDatabase::BZDB_MAXLOD		= std::string("_maxLOD");
-const std::string StateDatabase::BZDB_MOMENTUMLINACC  	= std::string("_momentumLinAcc");
-const std::string StateDatabase::BZDB_MOMENTUMANGACC  	= std::string("_momentumAngAcc");
-const std::string StateDatabase::BZDB_MGUNADVEL  	= std::string("_mGunAdVel");
-const std::string StateDatabase::BZDB_MGUNADRATE 	= std::string("_mGunAdRate");
-const std::string StateDatabase::BZDB_MGUNADLIFE 	= std::string("_mGunAdLife");
+const std::string StateDatabase::BZDB_MIRROR		= std::string("_mirror");
+const std::string StateDatabase::BZDB_MOMENTUMLINACC	= std::string("_momentumLinAcc");
+const std::string StateDatabase::BZDB_MOMENTUMANGACC	= std::string("_momentumAngAcc");
+const std::string StateDatabase::BZDB_MOMENTUMFRICTION	= std::string("_momentumFriction");
+const std::string StateDatabase::BZDB_MGUNADVEL		= std::string("_mGunAdVel");
+const std::string StateDatabase::BZDB_MGUNADRATE	= std::string("_mGunAdRate");
+const std::string StateDatabase::BZDB_MGUNADLIFE	= std::string("_mGunAdLife");
 const std::string StateDatabase::BZDB_MUZZLEFRONT	= std::string("_muzzleFront");
 const std::string StateDatabase::BZDB_MUZZLEHEIGHT	= std::string("_muzzleHeight");
+const std::string StateDatabase::BZDB_NOCLIMB		= std::string("_noClimb");
+const std::string StateDatabase::BZDB_NOSHADOWS		= std::string("_noShadows");
+const std::string StateDatabase::BZDB_NOSMALLPACKETS    = std::string("_noSmallPackets");
 const std::string StateDatabase::BZDB_NOTRESPONDINGTIME = std::string("_notRespondingTime");
 const std::string StateDatabase::BZDB_OBESEFACTOR       = std::string("_obeseFactor");
 const std::string StateDatabase::BZDB_PAUSEDROPTIME	= std::string("_pauseDropTime");
 const std::string StateDatabase::BZDB_POSITIONTOLERANCE = std::string("_positionTolerance");
-const std::string StateDatabase::BZDB_PYRBASE           = std::string("_pyrBase");
-const std::string StateDatabase::BZDB_PYRHEIGHT         = std::string("_pyrHeight");
-const std::string StateDatabase::BZDB_RELOADTIME        = std::string("_reloadTime");
-const std::string StateDatabase::BZDB_RFIREADVEL        = std::string("_rFireAdVel");
+const std::string StateDatabase::BZDB_PYRBASE		= std::string("_pyrBase");
+const std::string StateDatabase::BZDB_PYRHEIGHT		= std::string("_pyrHeight");
+const std::string StateDatabase::BZDB_RADARLIMIT	= std::string("_radarLimit");
+const std::string StateDatabase::BZDB_REJOINTIME	= std::string("_rejoinTime");
+const std::string StateDatabase::BZDB_RELOADTIME	= std::string("_reloadTime");
+const std::string StateDatabase::BZDB_RFIREADVEL	= std::string("_rFireAdVel");
 const std::string StateDatabase::BZDB_RFIREADRATE       = std::string("_rFireAdRate");
 const std::string StateDatabase::BZDB_RFIREADLIFE       = std::string("_rFireAdLife");
 const std::string StateDatabase::BZDB_SHIELDFLIGHT      = std::string("_shieldFlight");
@@ -118,31 +155,45 @@ const std::string StateDatabase::BZDB_SHOCKADLIFE       = std::string("_shockAdL
 const std::string StateDatabase::BZDB_SHOCKINRADIUS     = std::string("_shockInRadius");
 const std::string StateDatabase::BZDB_SHOCKOUTRADIUS    = std::string("_shockOutRadius");
 const std::string StateDatabase::BZDB_SHOTRADIUS	= std::string("_shotRadius");
-const std::string StateDatabase::BZDB_SHOTRANGE         = std::string("_shotRange");
-const std::string StateDatabase::BZDB_SHOTSPEED         = std::string("_shotSpeed");
+const std::string StateDatabase::BZDB_SHOTRANGE		= std::string("_shotRange");
+const std::string StateDatabase::BZDB_SHOTSPEED		= std::string("_shotSpeed");
 const std::string StateDatabase::BZDB_SHOTTAILLENGTH	= std::string("_shotTailLength");
+const std::string StateDatabase::BZDB_SHOTSKEEPVERTICALV= std::string("_shotsKeepVerticalVelocity");
 const std::string StateDatabase::BZDB_SRRADIUSMULT	= std::string("_srRadiusMult");
-const std::string StateDatabase::BZDB_TANKANGVEL        = std::string("_tankAngVel");
+const std::string StateDatabase::BZDB_SQUISHFACTOR	= std::string("_squishFactor");
+const std::string StateDatabase::BZDB_SQUISHTIME	= std::string("_squishTime");
+const std::string StateDatabase::BZDB_SYNCTIME		= std::string("_syncTime");
+const std::string StateDatabase::BZDB_SYNCLOCATION      = std::string("_syncLocation");
+const std::string StateDatabase::BZDB_TANKANGVEL	= std::string("_tankAngVel");
 const std::string StateDatabase::BZDB_TANKEXPLOSIONSIZE = std::string("_tankExplosionSize");
 const std::string StateDatabase::BZDB_TANKHEIGHT	= std::string("_tankHeight");
 const std::string StateDatabase::BZDB_TANKLENGTH	= std::string("_tankLength");
-const std::string StateDatabase::BZDB_TANKRADIUS        = std::string("_tankRadius");
-const std::string StateDatabase::BZDB_TANKSPEED         = std::string("_tankSpeed");
+const std::string StateDatabase::BZDB_TANKRADIUS	= std::string("_tankRadius");
+const std::string StateDatabase::BZDB_TANKSPEED		= std::string("_tankSpeed");
 const std::string StateDatabase::BZDB_TANKWIDTH		= std::string("_tankWidth");
 const std::string StateDatabase::BZDB_TARGETINGANGLE	= std::string("_targetingAngle");
+const std::string StateDatabase::BZDB_TELEBREADTH       = std::string("_teleportBreadth");
+const std::string StateDatabase::BZDB_TELEHEIGHT	= std::string("_teleportHeight");
 const std::string StateDatabase::BZDB_TELEPORTTIME      = std::string("_teleportTime");
+const std::string StateDatabase::BZDB_TELEWIDTH		= std::string("_teleportWidth");
 const std::string StateDatabase::BZDB_THIEFADLIFE	= std::string("_thiefAdLife");
 const std::string StateDatabase::BZDB_THIEFADRATE       = std::string("_thiefAdRate");
 const std::string StateDatabase::BZDB_THIEFADSHOTVEL    = std::string("_thiefAdShotVel");
 const std::string StateDatabase::BZDB_THIEFTINYFACTOR   = std::string("_thiefTinyFactor");
-const std::string StateDatabase::BZDB_THIEFVELAD        = std::string("_thiefVelAd");
+const std::string StateDatabase::BZDB_THIEFVELAD	= std::string("_thiefVelAd");
 const std::string StateDatabase::BZDB_THIEFDROPTIME     = std::string("_thiefDropTime");
-const std::string StateDatabase::BZDB_TINYFACTOR        = std::string("_tinyFactor");
+const std::string StateDatabase::BZDB_TINYFACTOR	= std::string("_tinyFactor");
+const std::string StateDatabase::BZDB_TRACKFADE		= std::string("_trackFade");
 const std::string StateDatabase::BZDB_UPDATETHROTTLERATE= std::string("_updateThrottleRate");
-const std::string StateDatabase::BZDB_VELOCITYAD        = std::string("_velocityAd");
-const std::string StateDatabase::BZDB_WALLHEIGHT        = std::string("_wallHeight");
+const std::string StateDatabase::BZDB_VELOCITYAD	= std::string("_velocityAd");
+const std::string StateDatabase::BZDB_WALLHEIGHT	= std::string("_wallHeight");
+const std::string StateDatabase::BZDB_WEAPONS		= std::string("_weapons");
 const std::string StateDatabase::BZDB_WIDEANGLEANG      = std::string("_wideAngleAng");
-const std::string StateDatabase::BZDB_WORLDSIZE         = std::string("_worldSize");
+const std::string StateDatabase::BZDB_WINGSGRAVITY      = std::string("_wingsGravity");
+const std::string StateDatabase::BZDB_WINGSJUMPCOUNT    = std::string("_wingsJumpCount");
+const std::string StateDatabase::BZDB_WINGSJUMPVELOCITY = std::string("_wingsJumpVelocity");
+const std::string StateDatabase::BZDB_WINGSSLIDETIME    = std::string("_wingsSlideTime");
+const std::string StateDatabase::BZDB_WORLDSIZE		= std::string("_worldSize");
 
 //
 // StateDatabase::Item
@@ -164,7 +215,7 @@ StateDatabase::Item::Item() : value(),
 //
 
 
-StateDatabase::StateDatabase() : debug(true)
+StateDatabase::StateDatabase() : debug(false)
 {
   // do nothing
 }
@@ -194,6 +245,26 @@ void			StateDatabase::set(const std::string& name,
   }
 }
 
+void		     StateDatabase::setInt(const std::string& name,
+					   const int& value,
+					   Permission access)
+{
+  set(name,TextUtils::format("%d",value),access);
+}
+
+void		    StateDatabase::setBool(const std::string& name,
+					   const bool& value,
+					   Permission access)
+{
+  set(name,value ? std::string("1") : std::string("0"),access);
+}
+
+void		   StateDatabase::setFloat(const std::string& name,
+					   const float& value,
+					   Permission access)
+{
+  set(name,TextUtils::format("%f",value),access);
+}
 
 void			StateDatabase::setPointer(const std::string& name,
 					   const void * value,
@@ -201,11 +272,7 @@ void			StateDatabase::setPointer(const std::string& name,
 {
   char address[32];
   memset(address, 0, 32);
-#ifdef _WIN32
-  _snprintf(address, 32, "%lu", (unsigned long)value);
-#else
   snprintf(address, 32, "%lu", (unsigned long)value);
-#endif
   std::string ssaddress = address;
   this->set(name, ssaddress, access);
 }
@@ -319,7 +386,7 @@ float			StateDatabase::eval(const std::string& name)
   VariableSet::iterator ins_it = variables.insert(name).first;
 
   Map::const_iterator index = items.find(name);
-  if (index == items.end() || !index->second.isSet) {
+  if (index == items.end() || !index->second.isSet || index->second.value.empty()) {
     variables.erase(ins_it);
     return NaN;
   }
@@ -334,6 +401,29 @@ float			StateDatabase::eval(const std::string& name)
   return retn;
 }
 
+int			StateDatabase::evalInt(const std::string& name)
+{
+  return (int)eval(name);
+}
+
+bool		StateDatabase::evalTriplet(const std::string& name, float data[4])
+{
+	if (!isSet(name) || !data)
+		return false;
+	if (sscanf(get(name).c_str(), "%f %f %f", data, data+1, data+2) != 3)
+	 return false;
+	return true;
+}
+
+bool		StateDatabase::evalPair(const std::string& name, float data[2])
+{
+	if (!isSet(name) || !data)
+		return false;
+	if (sscanf(get(name).c_str(), "%f %f", data, data+1) != 2)
+	 return false;
+	return true;
+}
+
 bool			StateDatabase::isTrue(const std::string& name) const
 {
   debugLookups(name);
@@ -429,13 +519,10 @@ void			StateDatabase::write(Callback callback, void* userData) const
   }
 }
 
-void                     StateDatabase::setDebug(bool print) {
+void		     StateDatabase::setDebug(bool print) {
   debug = print;
 }
 
-bool                     StateDatabase::getDebug() const {
-  return debug;
-}
 
 
 StateDatabase::ExpressionToken::ExpressionToken()
@@ -591,7 +678,7 @@ std::istream&operator >> (std::istream& src, StateDatabase::ExpressionToken& dst
 	dst.setOper(StateDatabase::ExpressionToken::rparen);
 	break;
     }
-  } else if((temp >= 'a' && temp <= 'z') || (temp >= 'A' && temp <= 'Z') || temp == '_') {
+  } else if ((temp >= 'a' && temp <= 'z') || (temp >= 'A' && temp <= 'Z') || temp == '_') {
     // variable (perhaps prefix with $?)
     tempname += temp;
     temp = src.peek();
@@ -738,7 +825,7 @@ std::string& operator >> (std::string& src, StateDatabase::Expression& dst)
 
 std::ostream& operator << (std::ostream& dst, const StateDatabase::Expression& src)
 {
-  if(src.size()) {
+  if (src.size()) {
     for (unsigned int i = 0; i < src.size() - 1; i++) {
       dst << src[i] << ' ';
     }
@@ -819,8 +906,8 @@ float StateDatabase::evaluate(Expression e) const
 	  case ExpressionToken::subtract:
 	    if (unary)
 	      tok.setNumber(-rvalue.getNumber());
-	  else
-	    tok.setNumber(lvalue.getNumber() - rvalue.getNumber());
+	    else
+	      tok.setNumber(lvalue.getNumber() - rvalue.getNumber());
 	    evaluationStack.push(tok);
 	    break;
 	  case ExpressionToken::multiply:
