@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -10,22 +10,27 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#include <math.h>
 #include "common.h"
+#include <math.h>
 #include "global.h"
+#include "Pack.h"
 #include "PyramidBuilding.h"
 #include "Intersect.h"
-#include "TriWallSceneNode.h"
-#include "QuadWallSceneNode.h"
-#include "BZDBCache.h"
+#include "MeshTransform.h"
 
-std::string		PyramidBuilding::typeName("PyramidBuilding");
+const char*		PyramidBuilding::typeName = "PyramidBuilding";
+
+PyramidBuilding::PyramidBuilding()
+{
+  // do nothing
+}
 
 PyramidBuilding::PyramidBuilding(const float* p, float a,
 				float w, float b, float h, bool drive, bool shoot) :
 				Obstacle(p, a, w, b, h,drive,shoot)
 {
-  // do nothing
+  finalize();
+  return;
 }
 
 PyramidBuilding::~PyramidBuilding()
@@ -33,12 +38,38 @@ PyramidBuilding::~PyramidBuilding()
   // do nothing
 }
 
-std::string		PyramidBuilding::getType() const
+void PyramidBuilding::finalize()
+{
+  Obstacle::setExtents();
+  return;
+}
+
+Obstacle* PyramidBuilding::copyWithTransform(const MeshTransform& xform) const
+{
+  float newPos[3], newSize[3], newAngle;
+  memcpy(newPos, pos, sizeof(float[3]));
+  memcpy(newSize, size, sizeof(float[3]));
+  newAngle = angle;
+
+  MeshTransform::Tool tool(xform);
+  bool flipped;
+  tool.modifyOldStyle(newPos, newSize, newAngle, flipped);
+
+  PyramidBuilding* copy =
+    new PyramidBuilding(newPos, newAngle, newSize[0], newSize[1], newSize[2],
+			driveThrough, shootThrough);
+
+  copy->ZFlip = !(getZFlip() == flipped);
+
+  return copy;
+}
+
+const char*		PyramidBuilding::getType() const
 {
   return typeName;
 }
 
-std::string		PyramidBuilding::getClassName() // const
+const char*		PyramidBuilding::getClassName() // const
 {
   return typeName;
 }
@@ -101,7 +132,7 @@ void			PyramidBuilding::get3DNormal(const float* p,
 		s * getWidth(), s * getBreadth(), n);
 
   // make sure we are not way above or way below it
-  // above is good so we can drive on it when it's fliped
+  // above is good so we can drive on it when it's flipped
   float top =  getPosition()[2]+getHeight();
   float bottom = getPosition()[2];
 
@@ -132,30 +163,38 @@ void			PyramidBuilding::get3DNormal(const float* p,
   }
 
   // now angle it due to slope of wall
-  // FIXME -- this assumes the pyramid has a square base!
-  const float h = 1.0f / hypotf(getHeight(), getWidth());
+  // we figure out if it was an X or Y wall that was hit
+  // by checking the normal returned from getNormalRect()
+  // FIXME -- fugly beyond belief
+  float baseLength = getWidth();
+  const float normalAngle = atan2f(n[1], n[0]);
+  const float rightAngle = fabsf(fmodf(normalAngle - getRotation() + (float)(M_PI/2.0), (float)M_PI));
+  if ((rightAngle < 0.1) || (rightAngle > (M_PI - 0.1))) {
+    baseLength = getBreadth();
+  }
+  const float h = 1.0f / hypotf(getHeight(), baseLength);
   n[0] *= h * getHeight();
   n[1] *= h * getHeight();
-  n[2]  = h * getWidth();
+  n[2]  = h * baseLength;
 
   if (this->getZFlip())
     n[2] *= -1;
 }
 
-bool			PyramidBuilding::isInside(const float* p,
-						float radius) const
+bool			PyramidBuilding::inCylinder(const float* p,
+						float radius, float height) const
 {
   // really rough -- doesn't decrease size with height
-  return (p[2] <= getHeight())
-  &&     ((p[2]+BZDBCache::tankHeight) >= getPosition()[2])
+  return (p[2] < (getPosition()[2] + getHeight()))
+  &&     ((p[2]+height) >= getPosition()[2])
   &&     testRectCircle(getPosition(), getRotation(), getWidth(), getBreadth(), p, radius);
 }
 
-bool			PyramidBuilding::isInside(const float* p, float a,
-						float dx, float dy) const
+bool			PyramidBuilding::inBox(const float* p, float a,
+						float dx, float dy, float height) const
 {
   // Tank is below pyramid ?
-  if (p[2] + BZDBCache::tankHeight < getPosition()[2])
+  if (p[2] + height < getPosition()[2])
     return false;
   // Tank is above pyramid ?
   if (p[2] >= getPosition()[2] + getHeight())
@@ -163,18 +202,25 @@ bool			PyramidBuilding::isInside(const float* p, float a,
   // Could be inside. Then check collision with the rectangle at object height
   // This is a rectangle reduced by shrinking but pass the height that we are
   // not so sure where collision can be
-  const float s = shrinkFactor(p[2], BZDBCache::tankHeight);
+  const float s = shrinkFactor(p[2], height);
   return testRectRect(getPosition(), getRotation(),
 		      s * getWidth(), s * getBreadth(), p, a, dx, dy);
 }
 
+bool			PyramidBuilding::inMovingBox(const float*, float,
+					     const float* p, float angle,
+					     float dx, float dy, float dz) const
+{
+  return inBox (p, angle, dx, dy, dz);
+}
+
 bool			PyramidBuilding::isCrossing(const float* p, float a,
-					float dx, float dy, float* plane) const
+					float dx, float dy, float height, float* plane) const
 {
   // if not inside or contained then not crossing
-  if (!isInside(p, a, dx, dy) ||
-	testRectInRect(getPosition(), getRotation(),
-			getWidth(), getBreadth(), p, a, dx, dy))
+  if (!inBox(p, a, dx, dy, height) ||
+      testRectInRect(getPosition(), getRotation(),
+		     getWidth(), getBreadth(), p, a, dx, dy))
     return false;
   if (!plane) return true;
 
@@ -238,7 +284,7 @@ bool			PyramidBuilding::getHitNormal(
   }
 
   normal[0] = normal[1] = 0;
-  if (flip && objHigh > oTop) {
+  if (flip && objHigh >= oTop) {
     // base of higher object is over the plateau
     normal[2] = 1;
     return true;
@@ -266,11 +312,6 @@ bool			PyramidBuilding::getHitNormal(
   return true;
 }
 
-ObstacleSceneNodeGenerator*	PyramidBuilding::newSceneNodeGenerator() const
-{
-  return new PyramidSceneNodeGenerator(this);
-}
-
 void			PyramidBuilding::getCorner(int index,
 						float* pos) const
 {
@@ -343,21 +384,21 @@ float			PyramidBuilding::shrinkFactor(float z,
   if (oHeight <= ZERO_TOLERANCE) {
     shrink = 1.0f;
   } else {
-  // Normalize heights
-  z /= oHeight;
+    // Normalize heights
+    z /= oHeight;
 
-  // if flipped the bigger intersection is at top of obiect
-  if (flip) {
-    // Normalize the object height, we have not done yet
-    z += height / oHeight;
-  }
+    // if flipped the bigger intersection is at top of the object
+    if (flip) {
+      // Normalize the object height, we have not done yet
+      z += height / oHeight;
+    }
 
-  // shrink is that
-  if (flip) {
-    shrink = z;
-  } else {
-    shrink = 1.0f - z;
-  }
+    // shrink is that
+    if (flip) {
+      shrink = z;
+    } else {
+      shrink = 1.0f - z;
+    }
   }
 
   // clamp in 0 .. 1
@@ -369,115 +410,87 @@ float			PyramidBuilding::shrinkFactor(float z,
   return shrink;
 }
 
-//
-// PyramidSceneNodeGenerator
-//
-
-PyramidSceneNodeGenerator::PyramidSceneNodeGenerator(
-				const PyramidBuilding* _pyramid) :
-				pyramid(_pyramid)
+bool			PyramidBuilding::isFlatTop() const
 {
-  // do nothing
+  return getZFlip();
 }
 
-PyramidSceneNodeGenerator::~PyramidSceneNodeGenerator()
+
+
+void* PyramidBuilding::pack(void* buf) const
 {
-  // do nothing
+  buf = nboPackVector(buf, pos);
+  buf = nboPackFloat(buf, angle);
+  buf = nboPackVector(buf, size);
+
+  unsigned char stateByte = 0;
+  stateByte |= isDriveThrough() ? _DRIVE_THRU : 0;
+  stateByte |= isShootThrough() ? _SHOOT_THRU : 0;
+  stateByte |= getZFlip() ? _FLIP_Z : 0;
+  buf = nboPackUByte(buf, stateByte);
+
+  return buf;
 }
 
-WallSceneNode*		PyramidSceneNodeGenerator::getNextNode(
-				float uRepeats, float vRepeats, bool lod)
+
+void* PyramidBuilding::unpack(void* buf)
 {
+  buf = nboUnpackVector(buf, pos);
+  buf = nboUnpackFloat(buf, angle);
+  buf = nboUnpackVector(buf, size);
 
-	bool isSquare = false;
+  unsigned char stateByte;
+  buf = nboUnpackUByte(buf, stateByte);
+  driveThrough = (stateByte & _DRIVE_THRU) != 0;
+  shootThrough = (stateByte & _SHOOT_THRU) != 0;
+  ZFlip = (stateByte & _FLIP_Z) != 0;
 
-	if (getNodeNumber() == 5) return NULL;
-
-  GLfloat base[3], sCorner[3], tCorner[3];
-  if (pyramid->getZFlip()){
-  switch (incNodeNumber()) {
-    case 1:
-      pyramid->getCorner(4, base);
-      pyramid->getCorner(1, sCorner);
-      pyramid->getCorner(0, tCorner);
-	  isSquare = false;
-      break;
-    case 2:
-      pyramid->getCorner(4, base);
-      pyramid->getCorner(2, sCorner);
-      pyramid->getCorner(1, tCorner);
-	  isSquare = false;
-      break;
-    case 3:
-      pyramid->getCorner(4, base);
-      pyramid->getCorner(3, sCorner);
-      pyramid->getCorner(2, tCorner);
-	  isSquare = false;
-      break;
-    case 4:
-      pyramid->getCorner(4, base);
-      pyramid->getCorner(0, sCorner);
-      pyramid->getCorner(3, tCorner);
-	  isSquare = false;
-      break;
-    case 5:
-      pyramid->getCorner(0, base);
-      pyramid->getCorner(1, sCorner);
-      pyramid->getCorner(3, tCorner);
-	  isSquare = true;
-      break;
-  }
-  }else{
-	  switch (incNodeNumber()) {
-    case 1:
-      pyramid->getCorner(0, base);
-      pyramid->getCorner(1, sCorner);
-      pyramid->getCorner(4, tCorner);
-	  isSquare = false;
-      break;
-    case 2:
-      pyramid->getCorner(1, base);
-      pyramid->getCorner(2, sCorner);
-      pyramid->getCorner(4, tCorner);
-	  isSquare = false;
-      break;
-    case 3:
-      pyramid->getCorner(2, base);
-      pyramid->getCorner(3, sCorner);
-      pyramid->getCorner(4, tCorner);
-	  isSquare = false;
-      break;
-    case 4:
-      pyramid->getCorner(3, base);
-      pyramid->getCorner(0, sCorner);
-      pyramid->getCorner(4, tCorner);
-	  isSquare = false;
-      break;
-    case 5:
-      pyramid->getCorner(0, base);
-      pyramid->getCorner(3, sCorner);
-      pyramid->getCorner(1, tCorner);
-	  isSquare = true;
-      break;
-  }
-  }
-
-  GLfloat sEdge[3];
-  GLfloat tEdge[3];
-  sEdge[0] = sCorner[0] - base[0];
-  sEdge[1] = sCorner[1] - base[1];
-  sEdge[2] = sCorner[2] - base[2];
-  tEdge[0] = tCorner[0] - base[0];
-  tEdge[1] = tCorner[1] - base[1];
-  tEdge[2] = tCorner[2] - base[2];
-
-  if(isSquare != true)
-	return new TriWallSceneNode(base, sEdge, tEdge, uRepeats, vRepeats, lod);
-  else
-	return new QuadWallSceneNode(base, sEdge, tEdge, uRepeats, vRepeats, lod);
+  finalize();
 
+  return buf;
 }
 
+
+int PyramidBuilding::packSize() const
+{
+  int fullSize = 0;
+  fullSize += sizeof(float[3]); // pos
+  fullSize += sizeof(float[3]); // size
+  fullSize += sizeof(float);    // rotation
+  fullSize += sizeof(uint8_t);  // state bits
+  return fullSize;
+}
+
+
+void PyramidBuilding::print(std::ostream& out, const std::string& indent) const
+{
+  out << indent << "pyramid" << std::endl;
+  const float *pos = getPosition();
+  out << indent << "  position " << pos[0] << " " << pos[1] << " "
+				 << pos[2] << std::endl;
+  float height = getHeight();
+  if (getZFlip()) {
+    height = -height;
+  }
+  out << indent << "  size " << getWidth() << " " << getBreadth()
+			     << " " << height << std::endl;
+  out << indent << "  rotation " << ((getRotation() * 180.0) / M_PI)
+				 << std::endl;
+  if (isDriveThrough() && isShootThrough()) {
+    out << indent << "  passable" << std::endl;
+  } else {
+    if (isDriveThrough()) {
+      out << indent << "  drivethrough" << std::endl;
+    }
+    if (isShootThrough()) {
+      out << indent << "  shootthrough" << std::endl;
+    }
+  }
+  out << indent << "end" << std::endl;
+  return;
+}
+
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
