@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993 - 2005 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -20,10 +20,6 @@
 #include "BzfDisplay.h"
 #include <X11/Xlib.h>
 
-#ifdef XIJOYSTICK
-#include <X11/extensions/XInput.h>
-#endif
-
 class BzfKeyEvent;
 class XDisplayMode;
 
@@ -50,32 +46,10 @@ class XDisplay : public BzfDisplay {
 	int		getScreen() const { return screen; }
 	Window		getRootWindow() const;
 
-#ifdef XIJOYSTICK
-	XDeviceInfo*	getDevices() const { return devices; }
-	int		getNDevices() const { return ndevices; }
-	void		setButtonPressType(int& type) {
-			  buttonPressType = type;
-			}
-	void		setButtonReleaseType(int& type) {
-			  buttonReleaseType = type;
-			}
-	int		getButtonPressType() const
-				{ return buttonPressType; }
-	int		getButtonReleaseType() const
-				{ return buttonReleaseType; }
-	int		mapButton(int button) const;
-#endif
-
       private:
 	int		refCount;
 	Display*	display;
 	int		screen;
-#ifdef XIJOYSTICK
-	XDeviceInfo*	devices;
-	int		ndevices;
-	int		buttonPressType;
-	int		buttonReleaseType;
-#endif
     };
     Rep*		getRep() const { return rep; }
 
